#include "mcp9808.h"

/* odczyt temperatury */
/* uwaga: wartosc zwracana jest przemnozona przez 100, np. 23,45*C => zwraca 2345 */

int16_t mcp9808_read_temp(void){
	
	uint32_t err = 0;
	uint8_t raw[2];
	
	/* uruchomienie interfejsu I2C */
	I2C_Cmd(I2C2, ENABLE);

	/* zerowanie flag przerwan */	
	I2C_ClearITPendingBit(I2C2, 0xFFFF);

	/* adres rejestru temperatury (TA) */	
	I2C_WriteDataCmd(I2C2, 0x05);


	/* przejscie w tryb odczytu */	
	I2C_WriteDataCmd(I2C2, I2C_DATA_CMD_READ);

	/* oczekiwanie na gotowosc */	
	while(	(I2C_GetFlagStatus(I2C2, I2C_FLAG_RFNE) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_TX_ABRT) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_SCL_STUCK_AT_LOW) == RESET));

	/* odczyt bajtu danych (MSB) */	
	raw[0] = I2C_ReadData(I2C2);
  

	/* po drugim odczycie ma byc wyslany sygnal stopu */	
	I2C_WriteDataCmd(I2C2, I2C_DATA_CMD_READ | I2C_DATA_CMD_STOP);

	/* oczekiwanie na gotowosc */	
	while(	(I2C_GetFlagStatus(I2C2, I2C_FLAG_RFNE) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_TX_ABRT) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_SCL_STUCK_AT_LOW) == RESET));

	/* odczyt bajtu danych (LSB) */	
	raw[1] = I2C_ReadData(I2C2);
	

	/* oczekiwanie na gotowosc */	
	while(	(I2C_GetFlagStatus(I2C2, I2C_FLAG_TFE) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_TX_ABRT) == RESET) &&
			(I2C_GetRawITStatus(I2C2, I2C_IT_SCL_STUCK_AT_LOW) == RESET));

	/* oczekiwanie na zakonczenie pracy maszyny stanow mastera */	
	while(I2C_GetFlagStatus(I2C2, I2C_FLAG_MST_ACTIVITY) != RESET);

	/* sprawdzenie flag ewentualnych bledow nadawania */	
	if(I2C_GetRawITStatus(I2C2, I2C_IT_TX_ABRT) != RESET) {

		/* zwrocenie informacji o przyczynie bledu nadawania */		
		err = I2C_GetTxAbortSource(I2C2);

		/* zerowanie flagi przerwania od bledu nadawania */		
		I2C_ClearITPendingBit(I2C2, I2C_IT_TX_ABRT);
	}
	
	else if(I2C_GetRawITStatus(I2C2, I2C_IT_SCL_STUCK_AT_LOW) != RESET) {

		/* arbitralnie przyjeta wartosc bledu oznaczajaca blokade SCL stanem niskim */		
		err = 0xFFFFFFFF;
			
		/* zerowanie flagi przerwania od bledu zegara */
		I2C_ClearITPendingBit(I2C2, I2C_IT_SCL_STUCK_AT_LOW);
	}
	
	
	I2C_Cmd(I2C2, DISABLE);

	/* zwrocenie arbitralnie przyjetej wartosci oznaczajacej blad odczytu */
	if(err) return 9999;	

	/* przeliczenie wartosci wyjsciowej na temperature w [*C]x100 */		
	raw[0] &= 0x1F;		
	return ((raw[0] * 1600) + (raw[1]*100 / 16));
}

